@@ -18,6 +18,7 @@ from toucan_connectors.oauth2_connector.oauth2connector import (
 )
 from toucan_connectors.toucan_connector import (
     ConnectorSecretsForm,
+    DataSlice,
     ToucanConnector,
     ToucanDataSource,
     strlist_to_enum,
@@ -66,25 +67,21 @@ class GithubDataSource(ToucanDataSource):
     organization: Optional[str] = Field(
         None, title='Organization', description='The organization to extract the data from'
     )
+    page_limit: int = Field(10, description='Limit of entries (default is 10 pages)', ge=0)
+    entities_limit: int = Field(
+        None,
+        title='Entities Limit',
+        description='Max Number of entities such as teams and repositories to extract',
+    )
 
     @classmethod
     def get_form(cls, connector: 'GithubConnector', current_config, **kwargs):
         """Retrieve a form filled with suggestions of available organizations."""
         # Always add the suggestions for the available organizations
         constraints = {}
-
         with suppress(Exception):
-            access_token = connector.get_access_token()
-
-            if not access_token:
-                raise NoCredentialsError('No credentials')
-
-            headers = {'Authorization': f'Bearer {access_token}'}
-            logging.getLogger(__name__).info('Retrieving organization')
-            data = requests.get(f'{BASE_ROUTE_REST}user/orgs', headers=headers).json()
-            available_organization = [str(x['login']) for x in data]
-            constraints['organization'] = strlist_to_enum('organization', available_organization)
-
+            available_organizations = connector.get_organizations()
+            constraints['organization'] = strlist_to_enum('organization', available_organizations)
         return create_model('FormSchema', **constraints, __base__=cls).schema()
 
 
@@ -120,6 +117,19 @@ class GithubConnector(ToucanConnector):
     def build_authorization_url(self, **kwargs):
         return self.__dict__['_oauth2_connector'].build_authorization_url(**kwargs)
 
+    def get_organizations(self):
+        """Retrieve a list of organizations available to the connector"""
+
+        access_token = self.get_access_token()
+
+        if not access_token:
+            raise NoCredentialsError('No credentials')
+
+        headers = {'Authorization': f'Bearer {access_token}'}
+        logging.getLogger(__name__).info('Retrieving organization')
+        data = requests.get(f'{BASE_ROUTE_REST}user/orgs', headers=headers).json()
+        return [str(x['login']) for x in data]
+
     def retrieve_tokens(self, authorization_response: str):
         """
         In the Github's oAuth2 authentication process, client_id & client_secret
@@ -132,7 +142,12 @@ class GithubConnector(ToucanConnector):
         return self.__dict__['_oauth2_connector'].get_access_token()
 
     def get_names(
-        self, client: GraphqlClient, organization: str, dataset: str, names=None, variables=None
+        self,
+        client: GraphqlClient,
+        organization: str,
+        dataset: str,
+        names=None,
+        variables=None,
     ) -> list:
         """
         Retrieve either repositories names or teams names
@@ -175,9 +190,9 @@ class GithubConnector(ToucanConnector):
         client: GraphqlClient,
         organization: str,
         dataset: str,
+        page_limit: int,
         variables=None,
         data_list=None,
-        page_limit=50,
         retrieved_pages=0,
         retries=0,
         retry_limit=4,
@@ -201,6 +216,7 @@ class GithubConnector(ToucanConnector):
         if data_list is None:
             data_list = []
         q = queries_funcs_pages[dataset](organization=organization, name=name)
+        await asyncio.sleep(0.200)
         data = await client.execute_async(query=q, variables=variables)
 
         try:
@@ -227,11 +243,12 @@ class GithubConnector(ToucanConnector):
                     variables=variables,
                     data_list=data_list,
                     retrieved_pages=retrieved_pages,
+                    page_limit=page_limit,
                 )
 
         except GithubError:
-            logging.getLogger(__name__).info('Retrying in 1 second')
-            await asyncio.sleep(1)
+            logging.getLogger(__name__).info('Retrying in 15 seconds')
+            await asyncio.sleep(15)
             retries += 1
             if retries <= retry_limit:
                 await self.get_pages(
@@ -243,6 +260,7 @@ class GithubConnector(ToucanConnector):
                     data_list=data_list,
                     retrieved_pages=retrieved_pages,
                     retries=retries,
+                    page_limit=page_limit,
                 )
             else:
                 raise GithubError('Max number of retries reached, aborting connection')
@@ -253,21 +271,34 @@ class GithubConnector(ToucanConnector):
         return data_list
 
     async def _fetch_data(
-        self, dataset: GithubDataSet, organization: str, client: GraphqlClient
+        self,
+        dataset: GithubDataSet,
+        organization: str,
+        client: GraphqlClient,
+        page_limit: int,
+        names_limit=None,
     ) -> pd.DataFrame:
         """
          Builds the coroutines ran by _retrieve_data
-        :param data_source  GithubDataSource, the GithubDataSource to query
+        :param dataset  GithubDataSet, the GithubDataSet to query
         :param organization a str representing the organization from
         which the data will be extracted
         :param client a GraphqlClient that will make the requests to Github's API
+        :param page_limit max number of pages to be retrieved by get_pages
+        :param names_limit number max of "names" (teams/repos) to extract the data from
         :return: a Pandas DataFrame of pull requests or team memberships
         """
         logging.getLogger(__name__).info(f'Starting fetch for {dataset}')
         names = self.get_names(client=client, organization=organization, dataset=dataset)
         subtasks = [
-            self.get_pages(name=name, client=client, dataset=dataset, organization=organization)
-            for name in names
+            self.get_pages(
+                name=name,
+                client=client,
+                dataset=dataset,
+                organization=organization,
+                page_limit=page_limit,
+            )
+            for name in names[:names_limit]
         ]
         unformatted_data = await asyncio.gather(*subtasks)
         return dataset_formatter[dataset]([e for sublist in unformatted_data for e in sublist])
@@ -289,9 +320,44 @@ class GithubConnector(ToucanConnector):
         client = GraphqlClient(BASE_ROUTE, headers)
         loop = get_loop()
         return loop.run_until_complete(
-            self._fetch_data(dataset=dataset, organization=organization, client=client)
+            self._fetch_data(
+                dataset=dataset,
+                organization=organization,
+                client=client,
+                page_limit=data_source.page_limit,
+                names_limit=data_source.entities_limit,
+            )
         )
 
+    def get_slice(
+        self,
+        data_source: GithubDataSource,
+        permissions: Optional[dict] = None,
+        offset: int = 0,
+        limit: Optional[int] = None,
+    ) -> DataSlice:
+        """
+        Method to retrieve a part of the data as a pandas dataframe
+        and the total size filtered with permissions
+
+        - offset is the index of the starting row
+        - limit is the number of pages to retrieve
+        Exemple: if offset = 5 and limit = 10 then 10 results are expected from 6th row
+        """
+        preview_datasource = GithubDataSource(
+            page_limit=1,
+            dataset=data_source.dataset,
+            domain=f'preview_{data_source.domain}',
+            name=data_source.name,
+            organization=data_source.organization,
+            entities_limit=3,
+        )
+        df = self.get_df(preview_datasource, permissions)
+        if limit is not None:
+            return DataSlice(df[offset : offset + limit], len(df))
+        else:
+            return DataSlice(df[offset:], len(df))
+
     def get_status(self) -> ConnectorStatus:
         """
         Test the Github's connexion.
