@@ -41,7 +41,7 @@ class MicroStrategyDataSource(ToucanDataSource):
         None,
         title='View filters',
         description='You can apply Microstrategy View Filters here. Please find configuration details in our '
-        '<a href="https://docs.toucantoco.com/concepteur/power-apps-with-data/02-add-data-to-small-app.html#microstrategy-connector" target="_blank"> '
+        '<a href="https://docs.toucantoco.com/concepteur/power-apps-with-data/01-add-data-to-small-app.html#microstrategy-connector" target="_blank"> '
         'documentation</a>',
     )
     offset: int = Field(
@@ -97,7 +97,8 @@ class MicroStrategyConnector(ToucanConnector):
         """Retrieves cube or report data, flattens return dataframe"""
         if data_source.dataset == Dataset.search:
             return self._retrieve_metadata(data_source)
-
+        if not self.password:
+            self.password = SecretStr('')
         client = Client(
             self.base_url, self.project_id, self.username, self.password.get_secret_value()
         )
